@@ -1,6 +1,10 @@
+from typing import Union, List, TYPE_CHECKING
+if TYPE_CHECKING:
+    from metacatalog.models import Entry
 import json
 from functools import wraps
 from datetime import datetime as dt
+from datetime import timedelta
 
 from sqlalchemy import Column, ForeignKey, CheckConstraint
 from sqlalchemy import Integer, String, DateTime, Numeric
@@ -52,9 +56,9 @@ class DataSourceType(Base):
     description = Column(String)
 
     # relationships
-    sources = relationship("DataSource", back_populates='type')
+    sources: List['DataSource'] = relationship("DataSource", back_populates='type')
 
-    def to_dict(self, deep=False) -> dict:
+    def to_dict(self, deep: bool = False) -> dict:
         """
         Return the model as a python dictionary.
 
@@ -88,7 +92,7 @@ class DataSourceType(Base):
 
         return d
 
-    def __str__(self):
+    def __str__(self) -> str:
         return '%s data source <ID=%d>' % (self.name, self.id)
 
 
@@ -123,10 +127,10 @@ class DataType(Base):
     description = Column(String, nullable=True)
 
     # relationships
-    sources = relationship("DataSource", back_populates='datatype')
-    children = relationship("DataType", backref=backref('parent', remote_side=[id]))
+    sources: List['DataSource'] = relationship("DataSource", back_populates='datatype')
+    children: List['DataType'] = relationship("DataType", backref=backref('parent', remote_side=[id]))
 
-    def to_dict(self, deep=False) -> dict:
+    def to_dict(self, deep: bool = False) -> dict:
         """
         Return the model as a python dictionary.
 
@@ -169,7 +173,7 @@ class DataType(Base):
 
         return d
 
-    def parent_list(self):
+    def parent_list(self) -> List['DataType']:
         """
         Returns an inheritance tree for the current datatype.
         If the list is empty, the current datatype is a
@@ -186,7 +190,7 @@ class DataType(Base):
 
         return parents
 
-    def children_list(self):
+    def children_list(self) -> List['DataType']:
         """
         Returns an dependency tree for the current datatype.
         If the list is empty, there are no child (inheriting)
@@ -258,7 +262,7 @@ class TemporalScale(Base):
     support = Column(Numeric, CheckConstraint('support >= 0'), nullable=False, default=1.0)
 
     # relationships
-    sources = relationship("DataSource", back_populates='temporal_scale')
+    sources: List['DataSource'] = relationship("DataSource", back_populates='temporal_scale')
 
     def __init__(self, *args, **kwargs):
         # handle resoultion
@@ -274,30 +278,30 @@ class TemporalScale(Base):
         super(TemporalScale, self).__init__(*args, **kwargs)
 
     @property
-    def resolution_timedelta(self):
+    def resolution_timedelta(self) -> pd.Timedelta:
         return pd.to_timedelta(self.resolution)
 
     @resolution_timedelta.setter
-    def resolution_timedelta(self, delta):
+    def resolution_timedelta(self, delta: Union[str, float, timedelta]):
         self.resolution = pd.to_timedelta(delta).isoformat()
 
     @property
-    def support_timedelta(self):
+    def support_timedelta(self) -> pd.Timedelta:
         return self.resolution_timedelta / 2
 
     @resolution_timedelta.setter
-    def support_timedelta(self, delta):
+    def support_timedelta(self, delta: Union[str, float, timedelta]):
         self.support = pd.to_timedelta(delta) / self.resolution_timedelta
 
     @property
-    def extent(self):
+    def extent(self) -> timedelta:
         return [self.observation_start, self.observation_end]
 
     @extent.setter
-    def extent(self, extent):
+    def extent(self, extent: List[dt]):
         self.observation_start, self.observation_end = extent
 
-    def to_dict(self, deep=False) -> dict:
+    def to_dict(self, deep: bool = False) -> dict:
         """
         Return the model as a python dictionary.
 
@@ -370,7 +374,7 @@ class SpatialScale(Base):
     support = Column(Numeric, CheckConstraint('support >= 0'), nullable=False, default=1.0)
 
     # relationships
-    sources = relationship("DataSource", back_populates='spatial_scale')
+    sources: List['DataSource'] = relationship("DataSource", back_populates='spatial_scale')
 
     @property
     def extent_shape(self):
@@ -392,7 +396,7 @@ class SpatialScale(Base):
             return '%d km' % (int((self.support * self.resolution) / 1000))
         return '%.1f m' % (self.support * self.resolution)
 
-    def to_dict(self, deep=False) -> dict:
+    def to_dict(self, deep: bool = False) -> dict:
         """
         Return the model as a python dictionary.
 
@@ -485,17 +489,17 @@ class DataSource(Base):
     lastUpdate = Column(DateTime, default=dt.utcnow, onupdate=dt.utcnow)
 
     # relationships
-    entries = relationship("Entry", back_populates='datasource')
-    type = relationship("DataSourceType", back_populates='sources')
-    datatype = relationship("DataType", back_populates='sources')
-    temporal_scale = relationship("TemporalScale", back_populates='sources')
-    spatial_scale = relationship("SpatialScale", back_populates='sources')
+    entries: List['Entry'] = relationship("Entry", back_populates='datasource')
+    type: 'DataSourceType' = relationship("DataSourceType", back_populates='sources')
+    datatype: 'DataType' = relationship("DataType", back_populates='sources')
+    temporal_scale: 'TemporalScale' = relationship("TemporalScale", back_populates='sources')
+    spatial_scale: 'SpatialScale' = relationship("SpatialScale", back_populates='sources')
 
     @classmethod
-    def is_valid(cls, ds):
+    def is_valid(cls, ds: 'DataSource') -> bool:
         return hasattr(ds, 'path') and isinstance(ds, DataSource)
 
-    def to_dict(self, deep=False) -> dict:
+    def to_dict(self, deep: bool = False) -> dict:
         """To dict
 
         Return the model as a python dictionary.
@@ -558,7 +562,7 @@ class DataSource(Base):
         else:
             return json.loads(self.args)
 
-    def save_args_from_dict(self, args_dict, commit=False):
+    def save_args_from_dict(self, args_dict: dict, commit: bool = False) -> None:
         """
         Save all given keyword arguments to the database.
         These are passed to the importer/adder functions as ``**kwargs``.
@@ -590,7 +594,7 @@ class DataSource(Base):
                 session.rollback()
                 raise e
 
-    def create_scale(self, resolution, extent, support, scale_dimension, commit=False):
+    def create_scale(self, resolution, extent, support, scale_dimension, commit: bool = False) -> None:
         """
         Create a new scale for the dataset
         """
@@ -619,5 +623,5 @@ class DataSource(Base):
                 session.rollback()
                 raise e
 
-    def __str__(self):
+    def __str__(self) -> str:
         return "%s data source at %s <ID=%d>" % (self.type.name, self.path, self.id)
