@@ -1,27 +1,32 @@
 """
 .. added: 0.1.6
 """
-from typing import Union
+from typing import Union, TYPE_CHECKING
+if TYPE_CHECKING:
+    from metacatalog.models import Entry, Thesaurus
+
 from sqlalchemy import Column, ForeignKey, UniqueConstraint
-from sqlalchemy import Integer, String, Text
+from sqlalchemy import Integer, String
 from sqlalchemy.orm import relationship
 from sqlalchemy.ext.mutable import MutableDict
 from sqlalchemy.dialects.postgresql import JSONB
+
 from metacatalog.db.base import Base
 
 
 class Detail(Base):
     """Entry detail
 
-    `Detail` data are optional key-value pairs that can be linked to
-    `metacatalo.models.Entry` records by `1:n` relationships. This is
-    vital metadata information that is specific to the `Entry` itself.
-    E.g. specific to the sensor or variable, but cannot be generalized
+    Details data are optional key-value pairs that can be linked to
+    :class:`Entry <metacatalog.models.Entry>` records by ``1:n`` relationships. 
+    This is vital metadata information that is specific to the 
+    :class:`Entry <metacatalog.models.Entry>` itself.
+    I.e. specific to the sensor or variable, but cannot be generalized
     to all kinds of `Entry` types.
 
     Details can be loaded as a python dict or be converted to
     text-based tables. A HTML or markdown table can e.g. be appended
-    to the `Entry.abstract` on export.
+    to the :class:`Entry.abstract <metacatalog.models.Entry.abstract>` on export.
 
     Since version 0.1.13, it is possible to link an existing
     :class:`Thesaurus <metacatalog.models.Thesaurus>` to the detail.
@@ -40,20 +45,23 @@ class Detail(Base):
         The key of the key vaule par. Maximum 20 letters,
         ideally no whitespaces.
         .. versionchanged:: 0.6.2
+
         Maximum letters changed from 20 to 60.
         Ideally, the key should come from a thesaurus available in the 
         database. In this case, also link the detail to the thesaurus
         via :attr:`thesaurus_id`.
     stem : str
-        Stemmed key using a `nltk.PorterStemmer`. The stemmed
+        Stemmed key using a ``nltk.PorterStemmer``. The stemmed
         key can be used to search for related keys
     title : str
         .. versionadded:: 0.6.2
+
         Optional longer and more descriptive title than :attr:`key`, 
         use this field e.g. if you used a thesaurus for attr:`key`, but 
         a longer or more precise title provides additional information.
     value : str, list
         .. versionchanged:: 0.3.0
+
         The actual value of this detail. This can be a string
         or a flat dictionary.
     description : str
@@ -63,6 +71,7 @@ class Detail(Base):
         can be omitted, if not applicable.
     thesaurus : metacatalog.models.Thesaurus
         .. versionadded:: 0.1.13
+
         Optional. If the detail :attr:`key` is described in a thesaurus or
         controlled dictionary list, you can link the thesaurus
         to the detail. Details with thesaurus information are
@@ -70,6 +79,7 @@ class Detail(Base):
         ``MD_MetadataExtensionInformation``.
     thesaurus_id : int
         .. versionadded:: 0.1.13
+
         Foreign key of the linked
         :class:`Thesaurus <metacatalog.models.Thesaurus>`.
 
@@ -90,8 +100,8 @@ class Detail(Base):
     thesaurus_id = Column(Integer, ForeignKey('thesaurus.id'))
 
     # relationships
-    entry = relationship("Entry", back_populates='details')
-    thesaurus = relationship("Thesaurus")
+    entry: 'Entry' = relationship("Entry", back_populates='details')
+    thesaurus: 'Thesaurus' = relationship("Thesaurus")
 
     def __init__(self, **kwargs):
         # handle values, if given
@@ -106,7 +116,7 @@ class Detail(Base):
         self.value = value
 
     @property
-    def value(self):
+    def value(self) -> Union[str, dict]:
         if '__literal__' in self.raw_value:
             return self.raw_value.get('__literal__')
         else:
@@ -120,10 +130,22 @@ class Detail(Base):
             new_val = new_value
         self.raw_value = new_val
 
-    def to_dict(self, deep=False):
+    def to_dict(self, deep: bool = False) -> dict:
         """
         Return the detail as JSON enabled dict.
 
+        Parameters
+        ----------
+        deep : bool
+            If True, the linked Entry will be loaded
+            into the dict as well, if False (default)
+            only the Entry.id and Entry.uuid are added.
+        
+        Returns
+        -------
+        d : dict
+            The dictionary representation of this Detail
+
         """
         d = dict(
             id=self.id,
@@ -143,7 +165,7 @@ class Detail(Base):
 
         return d
 
-    def __str__(self):
+    def __str__(self) -> str:
         if self.thesaurus is not None:
             return '%s = %s <%s>' % (self.key, self.value, self.thesaurus.name)
         else:
